package banco.conta;

import java.sql.*;
import java.util.HashMap;
import java.util.Collection;
import banco.util.ConnectionManager;
import banco.cliente.ClienteInexistenteException;
import banco.cliente.Cliente;

public class BDContas implements RepositorioContas {

    private static BDContas instance;
    private static Connection connection;
    private static Statement statement;

    private BDContas() {
    }

    public synchronized static BDContas getInstance() {
        if (instance == null) {
            instance = new BDContas();
        }
        return instance;
    }
       
    public void inserir(Conta conta) throws SQLException, ContaExistenteException {
        try {
        	if (this.existe(conta.getNumero())) {
        		throw new ContaExistenteException("BDContas => inserirConta  - Conta j cadastrada: " + conta.getNumero());
        	}
            Statement statement = ConnectionManager.preparaStatement();
            if (conta != null) {
            	int tipoConta = 0;
            	if (conta instanceof Poupanca)
            		tipoConta = 2;
            	else 
            		if (conta instanceof ContaBonificada) 
            			tipoConta = 3;
            		else 
            			if (conta instanceof Conta)
            				tipoConta = 1;
            	
            	statement.executeUpdate("INSERT INTO CONTAS VALUES ('" +
                                         conta.getNumero() + "', " +
                                         "'" + conta.getCliente().getCpf() + "', " +
                                         conta.getSaldo() + ", " +
                                         tipoConta + ")");
            }
        } catch (SQLException e) {
            throw new SQLException("BDContas => inserirConta: " + e);
	    }

    }

	public void remover(String numero) throws ContaInexistenteException, SQLException {
        try{
            Statement statement = ConnectionManager.preparaStatement();
		    int resultado = statement.executeUpdate("DELETE FROM CONTAS WHERE numero='" + numero + "'");
		} catch(SQLException e){
            throw new SQLException("BDContas => removerConta: " + e);
		} 
	}

	public Conta procurar(String numero) throws ContaInexistenteException, SQLException {
   	    Conta conta = null;
        try {
            Statement statement = ConnectionManager.preparaStatement();
            String sql = "SELECT * FROM CONTAS WHERE numero = '" + numero + "'";
            ResultSet resultset = statement.executeQuery(sql);
            if (resultset.next()) {
            	int tipo = resultset.getInt("tipoConta");
            	String numeroConta = resultset.getString("numero");
            	double saldo = resultset.getDouble("saldo");
            	Cliente cli = new Cliente(resultset.getString("Clientes_cpf"));
            	if (tipo == 1) 
            		conta = new Conta(numeroConta, saldo, cli);
            	else 
            		if (tipo == 2)
            			conta = new Poupanca(numeroConta, saldo, cli);
            		else
            			if (tipo == 3)
                			conta = new ContaBonificada(numeroConta, saldo, cli);
            } else {
                throw new ContaInexistenteException(numero);
            }
			resultset.close();
        } catch (SQLException e) {
			throw new SQLException("BDContas => procurarConta: " + e);
		} 
		return conta;
	}

	public void atualizar(Conta conta) throws ContaInexistenteException, SQLException {
	    try {
    	    Statement statement = ConnectionManager.preparaStatement();
            if (conta != null) {
                int resultado = statement.executeUpdate("UPDATE CONTAS SET saldo = " + conta.getSaldo() +
               	                                        " WHERE numero = '" + conta.getNumero() + "'");
                if (resultado == 0) {
                    throw new ContaInexistenteException(conta.getNumero());
                }
            }
        } catch (SQLException e) {
		    throw new SQLException("BDContas => atualizarConta: "+ e);
	    }
	}

	public boolean existe(String numero) throws SQLException {
        boolean resposta = false;
        try {
            Statement statement = ConnectionManager.preparaStatement();
            ResultSet resultset = statement.executeQuery("SELECT * FROM CONTAS WHERE numero = '" + numero + "'");
            resposta = resultset.next();
			resultset.close();
        } catch (SQLException e) {
			throw new SQLException("BDContas => existeConta: "+e);
		} 
        return resposta;
    }

	public Collection getContasCliente(Cliente cli) throws SQLException, ClienteInexistenteException, ContaExistenteException {
		ColecaoContas resposta = new ColecaoContas();
		try {
            Statement statement = ConnectionManager.preparaStatement();
            ResultSet resultset = statement.executeQuery("SELECT * FROM CONTAS WHERE Clientes_cpf ='" + cli.getCpf() + "'");
            while (resultset.next()) {
                Conta conta = new Conta(resultset.getString("numero"),
                                        resultset.getDouble("saldo"));
                conta.setCliente(cli);
                resposta.inserir(conta);
            }
		} catch (SQLException e) {
			throw new SQLException("BDContas => getContasCliente: " + e);
		} 
		return resposta.getContasCliente(cli);
    }
    
    public void liberaRecursos() {
		ConnectionManager.liberaRecursos();
	}

}