package banco.conta;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Collection;
import banco.cliente.Cliente;
import banco.cliente.ClienteInexistenteException;

public class ColecaoContas implements RepositorioContas {
	
	private HashMap contas;
	private static ColecaoContas instance;
	
	public ColecaoContas() {
		contas = new HashMap <String, Conta>();
	}
	
	public synchronized static ColecaoContas getInstance() {
		if (instance == null) {
			instance = new ColecaoContas();
		} 
		return instance;
	}
     
	public boolean existe(String cpf) {
		boolean achou = false;
		if (contas.containsKey(cpf)) {
			achou = true;
		}
		return achou;
	}
	
	public void atualizar(Conta c) 
    throws ContaInexistenteException {
		if (existe(c.getNumero())) {
			contas.remove(c);
			contas.put(c.getNumero(),c);
		} else {
			throw new ContaInexistenteException("Conta nao encontrada");
		}
	}
	
	public void inserir(Conta c) throws ContaExistenteException {
		if (!existe(c.getNumero())) {
			contas.put(c.getNumero(),c);
		} else {
			throw new ContaExistenteException("Conta j cadastrada");
		}
	}
	
	public Conta procurar(String num) throws ContaInexistenteException {
		Conta c = null;
		if (existe(num)) {
			c = (Conta) contas.get(num);
		} else {
			throw new ContaInexistenteException("Conta nao encontrada");
		}
		return c;
	}
	
	public void remover(String num) throws ContaInexistenteException {
		if (existe(num)) {
			contas.remove(num);
		} else {
			throw new ContaInexistenteException("Conta nao encontrada");
		}
	}
	
	public Collection getContasCliente(Cliente cli) throws SQLException, ClienteInexistenteException, ContaExistenteException {
		return this.contas.values();
	}
	
	
	public void liberaRecursos() {
		contas = null;
	}
	
}
