package banco.fachada;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Collection;
import banco.cliente.Cliente;
import banco.cliente.ClienteExistenteException;
import banco.cliente.ClienteInexistenteException;
import banco.cliente.ColecaoClientes;
import banco.conta.ColecaoContas;
import banco.conta.Conta;
import banco.conta.ContaExistenteException;
import banco.conta.ContaInexistenteException;
import banco.conta.SaldoInsuficienteException;
import banco.conta.RepositorioContas;
import banco.conta.ArrayContas;
import banco.cliente.RepositorioClientes;
import banco.conta.BDContas;
import banco.cliente.BDClientes;


public class Fachada {

  private static Fachada instancia;
  //private ArrayContas contas;
  //private ArrayClientes clientes;
  private RepositorioContas   contas;
  private RepositorioClientes clientes;
  
  protected Fachada() {
	  initCadastros();
  }

  private void initCadastros() {
	//contas = new ArrayContas();
	//clientes = new ArrayClientes();
	//contas = ColecaoContas.getInstance();
	//clientes = ColecaoClientes.getInstance();  
	contas = BDContas.getInstance();
    clientes = BDClientes.getInstance();
    
  }


  public static Fachada obterInstancia() {
	 if (instancia == null) {
	   instancia = new Fachada();
	 }
	 return instancia;
  }

  //metodos par manipular clientes
  public void atualizarCliente(Cliente c) throws ClienteInexistenteException, SQLException{
	  clientes.atualizar(c);
  }
  public Cliente procurarCliente(String cpf) throws ClienteInexistenteException, SQLException{
	return clientes.procurar(cpf);
  }
  public void inserirCliente(Cliente c) throws ClienteExistenteException, SQLException{
	clientes.inserir(c);
  }
  public void removerCliente(String cpf) throws ClienteInexistenteException, SQLException{
	  clientes.remover(cpf);
  }

  //metodos para manipular contas
  public void atualizarConta(Conta c) throws ContaInexistenteException, SQLException{
	contas.atualizar(c);
  }
  public Conta procurarConta(String n) throws ContaInexistenteException, SQLException{
	  return contas.procurar(n);
  }
  public void inserirConta(Conta c)
    throws ContaExistenteException, ClienteInexistenteException, SQLException{

    Cliente cli = c.getCliente();
    if(cli != null){
      clientes.procurar(cli.getCpf());
      contas.inserir(c);
    }else {
      throw new IllegalArgumentException("cliente nulo");
    }
  }

  public void removerConta(String n) throws ContaInexistenteException, SQLException{
	  contas.remover(n);
  }

  public void creditar(String n, double v) throws ContaInexistenteException, SQLException{
    Conta c = contas.procurar(n);
    c.creditar(v);
    contas.atualizar(c);
  }

  public void debitar(String n, double v) throws ContaInexistenteException,
    SaldoInsuficienteException, SQLException {
    Conta c = contas.procurar(n);
    c.debitar(v);
    contas.atualizar(c);
  }

  public void transferir(String origem, String destino, double valor)
    throws ContaInexistenteException, SaldoInsuficienteException, SQLException {
	  Conta contaDe = contas.procurar(origem);
      Conta contaPara = contas.procurar(destino);
      contaDe.debitar(valor);
      contaPara.creditar(valor);
      contas.atualizar(contaDe);
      contas.atualizar(contaPara);

  }
  
  public Collection getContasCliente(String cpf) throws SQLException, ClienteInexistenteException, ContaExistenteException {
	  Cliente c = clientes.procurar(cpf);
	  return contas.getContasCliente(c);
  }
  
  public void liberaRecursos() {
	  contas.liberaRecursos();
	  clientes.liberaRecursos();
	  
  }

}
